@@ -38,34 +38,39 @@ class EmaneRfPipeModel(EmaneModel):
     _confmatrix_mac_base = [
         ("enablepromiscuousmode", coreapi.CONF_DATA_TYPE_BOOL, '0',
          'True,False', 'enable promiscuous mode'),
-        ("datarate", coreapi.CONF_DATA_TYPE_UINT32, '1M', 
+        ("datarate", coreapi.CONF_DATA_TYPE_UINT32, '1M',
          '', 'data rate (bps)'),
-        ("jitter", coreapi.CONF_DATA_TYPE_FLOAT, '0.0', 
-         '', 'transmission jitter (usec)'),
-        ("delay", coreapi.CONF_DATA_TYPE_FLOAT, '0.0', 
-         '', 'transmission delay (usec)'),
-        ("flowcontrolenable", coreapi.CONF_DATA_TYPE_BOOL, '0', 
+        ("flowcontrolenable", coreapi.CONF_DATA_TYPE_BOOL, '0',
          'On,Off', 'enable traffic flow control'),
-        ("flowcontroltokens", coreapi.CONF_DATA_TYPE_UINT16, '10', 
+        ("flowcontroltokens", coreapi.CONF_DATA_TYPE_UINT16, '10',
          '', 'number of flow control tokens'),
         ("pcrcurveuri", coreapi.CONF_DATA_TYPE_STRING,
          '%s/rfpipepcr.xml' % xml_path,
          '', 'SINR/PCR curve file'),
     ]
     _confmatrix_mac_081 = [
+        ("jitter", coreapi.CONF_DATA_TYPE_FLOAT, '0.0',
+         '', 'transmission jitter (usec)'),
+        ("delay", coreapi.CONF_DATA_TYPE_FLOAT, '0.0',
+         '', 'transmission delay (usec)'),
         ("transmissioncontrolmap", coreapi.CONF_DATA_TYPE_STRING, '',
          '', 'tx control map (nem:rate:freq:tx_dBm)'),
         ("enabletighttiming", coreapi.CONF_DATA_TYPE_BOOL, '0',
          'On,Off', 'enable tight timing for pkt delay'),
     ]
-    _confmatrix_mac_091 = []
+    _confmatrix_mac_091 = [
+        ("jitter", coreapi.CONF_DATA_TYPE_FLOAT, '0.0',
+         '', 'transmission jitter (sec)'),
+        ("delay", coreapi.CONF_DATA_TYPE_FLOAT, '0.0',
+         '', 'transmission delay (sec)'),
+    ]
     if 'EventService' in globals():
         _confmatrix_mac = _confmatrix_mac_base + _confmatrix_mac_091
     else:
         _confmatrix_mac = _confmatrix_mac_base + _confmatrix_mac_081
-    
+
     # PHY parameters from Universal PHY
-    _confmatrix_phy = EmaneUniversalModel._confmatrix 
+    _confmatrix_phy = EmaneUniversalModel._confmatrix
 
     _confmatrix = _confmatrix_mac + _confmatrix_phy
 
@@ -86,6 +91,7 @@ class EmaneRfPipeModel(EmaneModel):
         nemdoc = e.xmldoc("nem")
         nem = nemdoc.getElementsByTagName("nem").pop()
         nem.setAttribute("name", "RF-PIPE NEM")
+        e.appendtransporttonem(nemdoc, nem, self.objid, ifc)
         mactag = nemdoc.createElement("mac")
         mactag.setAttribute("definition", self.macxmlname(ifc))
         nem.appendChild(mactag)
@@ -102,7 +108,7 @@ class EmaneRfPipeModel(EmaneModel):
         mac = macdoc.getElementsByTagName("mac").pop()
         mac.setAttribute("name", "RF-PIPE MAC")
         mac.setAttribute("library", "rfpipemaclayer")
-        if e.version != e.EMANE091 and \
+        if e.version < e.EMANE091 and \
            self.valueof("transmissioncontrolmap", values) is "":
             macnames.remove("transmissioncontrolmap")
         # EMANE 0.7.4 support
