@@ -20,21 +20,27 @@ from core.coreobj import PyCoreNode
 class CtrlNet(LxBrNet):
     policy = "ACCEPT"
     CTRLIF_IDX_BASE = 99  # base control interface index
-
+    DEFAULT_PREFIX_LIST = ["172.16.0.0/24 172.16.1.0/24 172.16.2.0/24 172.16.3.0/24 172.16.4.0/24",
+                           "172.17.0.0/24 172.17.1.0/24 172.17.2.0/24 172.17.3.0/24 172.17.4.0/24",
+                           "172.18.0.0/24 172.18.1.0/24 172.18.2.0/24 172.18.3.0/24 172.18.4.0/24",
+                           "172.19.0.0/24 172.19.1.0/24 172.19.2.0/24 172.19.3.0/24 172.19.4.0/24"]
+    
     def __init__(self, session, objid = "ctrlnet", name = None,
-                 verbose = False, netid = 1, prefix = None,
+                 verbose = False, prefix = None,  
                  hostid = None, start = True, assign_address = True,
-                 updown_script = None):
-        if not prefix:
-            prefix = "172.16.%d.0/24" % netid
+                 updown_script = None, serverintf = None):
         self.prefix = IPv4Prefix(prefix)
         self.hostid = hostid
         self.assign_address = assign_address
         self.updown_script = updown_script
+        self.serverintf = serverintf
         LxBrNet.__init__(self, session, objid = objid, name = name,
                          verbose = verbose, start = start)
 
     def startup(self):
+        if self.detectoldbridge():
+            return
+        
         LxBrNet.startup(self)
         if self.hostid:
             addr = self.prefix.addr(self.hostid)
@@ -51,8 +57,58 @@ class CtrlNet(LxBrNet):
             self.info("interface %s updown script '%s startup' called" % \
                       (self.brname, self.updown_script))
             check_call([self.updown_script, self.brname, "startup"])
+        if self.serverintf is not None:
+            try:
+                check_call([BRCTL_BIN, "addif", self.brname, self.serverintf])
+                check_call([IP_BIN, "link", "set", self.serverintf, "up"])
+            except Exception, e:
+                self.exception(coreapi.CORE_EXCP_LEVEL_FATAL, self.brname,
+                               "Error joining server interface %s to controlnet bridge %s: %s" % \
+                               (self.serverintf, self.brname, e))
+                
 
+    def detectoldbridge(self):
+        ''' Occassionally, control net bridges from previously closed sessions are not cleaned up.
+        Check if there are old control net bridges and delete them
+        ''' 
+        retstat, retstr = cmdresult([BRCTL_BIN,'show'])
+        if retstat != 0:
+            self.exception(coreapi.CORE_EXCP_LEVEL_FATAL, None,
+                           "Unable to retrieve list of installed bridges")
+        lines = retstr.split('\n')
+        for line in lines[1:]:
+            cols = line.split('\t')
+            oldbr = cols[0]
+            flds = cols[0].split('.')
+            if len(flds) == 3:
+                if flds[0] == 'b' and flds[1] == self.objid:
+                    self.session.exception(coreapi.CORE_EXCP_LEVEL_FATAL, "CtrlNet.startup()", None,
+                                           "Error: An active control net bridge (%s) found. "\
+                                           "An older session might still be running. " \
+                                           "Stop all sessions and, if needed, delete %s to continue." % \
+                                           (oldbr, oldbr))
+                    return True
+                    '''
+                    # Do this if we want to delete the old bridge
+                    self.warn("Warning: Old %s bridge found: %s" % (self.objid, oldbr))
+                    try:
+                        check_call([BRCTL_BIN, 'delbr', oldbr])
+                    except Exception, e:
+                        self.exception(coreapi.CORE_EXCP_LEVEL_ERROR, oldbr,
+                                       "Error deleting old bridge %s" % oldbr)
+                    self.info("Deleted %s" % oldbr)
+                    '''
+        return False
+        
     def shutdown(self):
+        if self.serverintf is not None:
+            try:
+                check_call([BRCTL_BIN, "delif", self.brname, self.serverintf])
+            except Exception, e:
+                self.exception(coreapi.CORE_EXCP_LEVEL_ERROR, self.brname,
+                               "Error deleting server interface %s to controlnet bridge %s: %s" % \
+                               (self.serverintf, self.brname, e))
+            
         if self.updown_script is not None:
             self.info("interface %s updown script '%s shutdown' called" % \
                       (self.brname, self.updown_script))
@@ -107,6 +163,9 @@ class PtpNet(LxBrNet):
 
         tlvdata += coreapi.CoreLinkTlv.pack(coreapi.CORE_TLV_LINK_IF1NUM, \
                                             if1.node.getifindex(if1))
+        if if1.hwaddr:
+            tlvdata += coreapi.CoreLinkTlv.pack(coreapi.CORE_TLV_LINK_IF1MAC,
+                                                if1.hwaddr)
         for addr in if1.addrlist:
             (ip, sep, mask)  = addr.partition('/')
             mask = int(mask)
@@ -125,6 +184,9 @@ class PtpNet(LxBrNet):
 
         tlvdata += coreapi.CoreLinkTlv.pack(coreapi.CORE_TLV_LINK_IF2NUM, \
                                             if2.node.getifindex(if2))
+        if if2.hwaddr:
+            tlvdata += coreapi.CoreLinkTlv.pack(coreapi.CORE_TLV_LINK_IF2MAC,
+                                                if2.hwaddr)
         for addr in if2.addrlist:
             (ip, sep, mask)  = addr.partition('/')
             mask = int(mask)
@@ -168,7 +230,7 @@ class HubNode(LxBrNet):
     apitype = coreapi.CORE_NODE_HUB
     policy = "ACCEPT"
     type = "hub"
-    
+
     def __init__(self, session, objid = None, name = None, verbose = False,
                         start = True):
         ''' the Hub node forwards packets to all bridge ports by turning off
@@ -184,7 +246,7 @@ class WlanNode(LxBrNet):
     linktype = coreapi.CORE_LINK_WIRELESS
     policy = "DROP"
     type = "wlan"
-    
+
     def __init__(self, session, objid = None, name = None, verbose = False,
                         start = True, policy = None):
         LxBrNet.__init__(self, session, objid, name, verbose, start, policy)
@@ -192,7 +254,7 @@ class WlanNode(LxBrNet):
         self.model = None
         # mobility model such as scripted
         self.mobility = None
-        
+
     def attach(self, netif):
         LxBrNet.attach(self, netif)
         if self.model:
@@ -203,7 +265,7 @@ class WlanNode(LxBrNet):
             # invokes any netif.poshook
             netif.setposition(x, y, z)
             #self.model.setlinkparams()
-        
+
     def setmodel(self, model, config):
         ''' Mobility and wireless model.
         '''
@@ -222,7 +284,7 @@ class WlanNode(LxBrNet):
         elif model._type == coreapi.CORE_TLV_REG_MOBILITY:
             self.mobility = model(session=self.session, objid=self.objid,
                                verbose=self.verbose, values=config)
-                               
+
     def updatemodel(self, model_name, values):
         ''' Allow for model updates during runtime (similar to setmodel().)
         '''
@@ -241,7 +303,7 @@ class WlanNode(LxBrNet):
                         (x,y,z) = netif.node.position.get()
                         netif.poshook(netif, x, y, z)
             self.model.setlinkparams()
-        
+
     def tolinkmsgs(self, flags):
         msgs = LxBrNet.tolinkmsgs(self, flags)
         if self.model:
@@ -255,7 +317,7 @@ class RJ45Node(PyCoreNode, PyCoreNetIf):
     '''
     apitype = coreapi.CORE_NODE_RJ45
     type = "rj45"
-    
+
     def __init__(self, session, objid = None, name = None, mtu = 1500,
                  verbose = False, start = True):
         PyCoreNode.__init__(self, session, objid, name, verbose=verbose,
@@ -283,7 +345,7 @@ class RJ45Node(PyCoreNode, PyCoreNetIf):
                       (IP_BIN, self.localname))
             return
         self.up = True
-        
+
     def shutdown(self):
         ''' Bring the interface down. Remove any addresses and queuing
             disciplines.
@@ -295,16 +357,16 @@ class RJ45Node(PyCoreNode, PyCoreNetIf):
         mutecall([TC_BIN, "qdisc", "del", "dev", self.localname, "root"])
         self.up = False
         self.restorestate()
-        
+
     def attachnet(self, net):
         PyCoreNetIf.attachnet(self, net)
-        
+
     def detachnet(self):
         PyCoreNetIf.detachnet(self)
 
     def newnetif(self, net = None, addrlist = [], hwaddr = None,
-                 ifindex = None, ifname = None):        
-        ''' This is called when linking with another node. Since this node 
+                 ifindex = None, ifname = None):
+        ''' This is called when linking with another node. Since this node
             represents an interface, we do not create another object here,
             but attach ourselves to the given network.
         '''
@@ -325,7 +387,7 @@ class RJ45Node(PyCoreNode, PyCoreNetIf):
             return ifindex
         finally:
             self.lock.release()
-    
+
     def delnetif(self, ifindex):
         if ifindex is None:
             ifindex = 0
@@ -349,12 +411,12 @@ class RJ45Node(PyCoreNode, PyCoreNetIf):
         if ifindex == self.ifindex:
             return self
         return None
-    
+
     def getifindex(self, netif):
         if netif != self:
             return None
         return self.ifindex
-        
+
     def addaddr(self, addr):
         if self.up:
             check_call([IP_BIN, "addr", "add", str(addr), "dev", self.name])
@@ -364,7 +426,7 @@ class RJ45Node(PyCoreNode, PyCoreNetIf):
         if self.up:
             check_call([IP_BIN, "addr", "del", str(addr), "dev", self.name])
         PyCoreNetIf.deladdr(self, addr)
-        
+
     def savestate(self):
         ''' Save the addresses and other interface state before using the
         interface for emulation purposes. TODO: save/restore the PROMISC flag
@@ -395,7 +457,7 @@ class RJ45Node(PyCoreNode, PyCoreNetIf):
                 if items[1][:4] == "fe80":
                     continue
                 self.old_addrs.append((items[1], None))
-                    
+
     def restorestate(self):
         ''' Restore the addresses and other interface state after using it.
         '''
@@ -404,19 +466,19 @@ class RJ45Node(PyCoreNode, PyCoreNetIf):
                 check_call([IP_BIN, "addr", "add", addr[0], "dev",
                             self.localname])
             else:
-                check_call([IP_BIN, "addr", "add", addr[0], "brd", addr[1], 
+                check_call([IP_BIN, "addr", "add", addr[0], "brd", addr[1],
                             "dev", self.localname])
         if self.old_up:
             check_call([IP_BIN, "link", "set", self.localname, "up"])
-            
+
     def setposition(self, x=None, y=None, z=None):
         ''' Use setposition() from both parent classes.
         '''
         PyCoreObj.setposition(self, x, y, z)
         # invoke any poshook
         PyCoreNetIf.setposition(self, x, y, z)
-               
-            
+
+
 
 
 
