@@ -3,6 +3,20 @@
 # See the LICENSE file included in this distribution.
 #
 
+set vtysh_cmd vtysh
+set vtysh_search_path {/usr/bin /usr/local/bin /usr/lib/quagga}
+
+set vtysh [auto_execok $vtysh_cmd]
+if {$vtysh == ""} {
+    set vtysh $vtysh_cmd
+    foreach p $vtysh_search_path {
+	if {[file executable $p/$vtysh_cmd]} {
+	    set vtysh $p/$vtysh_cmd
+	    break
+	}
+    }
+}
+
 #
 # Widgets are defined in this array.
 # widget array: name, {config, init, periodic, move}
@@ -19,19 +33,19 @@ array set widgets {
 #	{ widget_cpu_config widget_cpu_init widget_cpu_periodic widget_cpu_move }
 
 # Common Observer Widget definitions
-set widgets_obs_quagga {
+set widgets_obs_quagga [subst {
     5 
-    {{OSPFv2 neighbors} {vtysh -c {show ip ospf neighbor}}}
+    {{OSPFv2 neighbors} {$vtysh -c {show ip ospf neighbor}}}
 
     6
-    {{OSPFv3 neighbors} {vtysh -c {show ipv6 ospf6 neighbor}}}
-
-    12
-    {{OSPFv3 MDR level} {vtysh -c {show ipv6 ospf6 mdrlevel}}}
+    {{OSPFv3 neighbors} {$vtysh -c {show ipv6 ospf6 neighbor}}}
 
     13
-    {{PIM neighbors} {vtysh -c {show ip pim neighbor}}}
-}
+    {{OSPFv3 MDR level} {$vtysh -c {show ipv6 ospf6 mdrlevel}}}
+
+    14
+    {{PIM neighbors} {$vtysh -c {show ip pim neighbor}}}
+}]
 
 # Observer Widget definitions for FreeBSD
 array set widgets_obs_bsd $widgets_obs_quagga
@@ -79,6 +93,8 @@ array set widgets_obs_linux {
 	{ "firewall rules" "/sbin/iptables -L" }
 	11
 	{ "IPSec policies" "setkey -DP" }
+	12
+        { "docker logs" "bash -c 'docker logs $(docker ps -q) | tail -20'"}
 }
 
 set widget_loop_ID -1
@@ -908,7 +924,7 @@ proc getstats_link_ifname { link } {
     # TODO: need to determine session number used by daemon
     #       instead this uses a '*' character for a regexp match against
     #       the interfaces in /proc/net/dev
-    set ifname "n$node_num\\.$ifname\\.*"
+    set ifname "veth$node_num\\.[string range $ifname 3 end]\\.*"
     return $ifname
 }
 
@@ -1889,7 +1905,7 @@ proc widget_adjacency_config_apply { wi } {
 }
 
 proc get_router_id {node} {
-    global oper_mode
+    global oper_mode vtysh
 
     # search custom-config
     if { [getCustomEnabled $node] == true } {
@@ -1914,16 +1930,28 @@ proc get_router_id {node} {
 	    return [string range $line 11 end]
 	}
     }
-
+    if {[lsearch [getNodeServices $node true] "OLSR"] != -1 } {
+        
+        set sock [lindex [getEmulPlugin $node] 2]
+        set exec_num [newExecCallbackRequest adjacencyrouterid]
+        set cmd "nrlConsole.py ${node}_olsr i"
+        sendExecMessage $sock $node $cmd $exec_num 0x30
+        return ""
+    } elseif {[lsearch [getNodeServices $node true] "OLSRv2"] != -1 } {
+        set sock [lindex [getEmulPlugin $node] 2]
+        set exec_num [newExecCallbackRequest adjacencyrouterid]
+        set cmd "nrlConsole.py ${node}_olsrv2 i"
+        sendExecMessage $sock $node $cmd $exec_num 0x30
+        return ""
+    }
     if { $oper_mode != "exec" }  { return }
-
     # use exec message here for retrieving router ID
     set sock [lindex [getEmulPlugin $node] 2]
     set exec_num [newExecCallbackRequest adjacencyrouterid]
-    set cmd "vtysh -c 'show ipv6 ospf6'"
+    set cmd "$vtysh -c 'show ipv6 ospf6'"
     sendExecMessage $sock $node $cmd $exec_num 0x30
     set exec_num [newExecCallbackRequest adjacencyrouterid]
-    set cmd "vtysh -c 'show ip ospf'"
+    set cmd "$vtysh -c 'show ip ospf'"
     sendExecMessage $sock $node $cmd $exec_num 0x30
     return ""
 }
@@ -1932,7 +1960,7 @@ proc get_router_id {node} {
 # Adjacency widget de/initialization
 #
 proc widget_adjacency_init {command} {
-    global enable_Adjacency enable_Adjacency_v2 enable_Adjacency_v3
+    global enable_Adjacency enable_Adjacency_OSPFv2 enable_Adjacency_OSPFv3 enable_Adjacency_OLSR enable_Adjacency_OLSRv2
     global node_list adjacency_cache adjacency_config adjacency_lock
     global g_execRequests
     set c .c
@@ -1941,26 +1969,42 @@ proc widget_adjacency_init {command} {
     array set g_execRequests [list adjacency ""]
 
     # Menu item selected on/off
-    if { $command == "menu2" || $command == "menu3" } {
+    if { $command == "menu2" || $command == "menu3" || $command == "menu4" || $command == "menu5" } {
 	# set global enable flag for v2/v3 adjacency display
 	set enable_Adjacency \
-	    [expr {$enable_Adjacency_v2 | $enable_Adjacency_v3}]
+	    [expr {$enable_Adjacency_OSPFv2 | $enable_Adjacency_OSPFv3 | $enable_Adjacency_OLSR | $enable_Adjacency_OLSRv2}]
 	# toggle other OSPFv2/v3 menu items off
-	if { $command == "menu2" && $enable_Adjacency_v2 } {
-	    set enable_Adjacency_v3 0
+	if { $command == "menu2" && $enable_Adjacency_OSPFv2 } {
+	    set enable_Adjacency_OSPFv3 0
+            set enable_Adjacency_OLSR 0
+            set enable_Adjacency_OLSRv2 0
 	    set adjacency_config(proto) "ip ospf"
-	} elseif { $command == "menu3" && $enable_Adjacency_v3 } {
-	    set enable_Adjacency_v2 0
+	} elseif { $command == "menu3" && $enable_Adjacency_OSPFv3 } {
+	    set enable_Adjacency_OSPFv2 0
+            set enable_Adjacency_OLSR 0
+            set enable_Adjacency_OLSRv2 0
 	    set adjacency_config(proto) "ipv6 ospf6"
-	}
+	} elseif { $command == "menu4" && $enable_Adjacency_OLSR } {
+            set enable_Adjacency_OSPFv2 0
+            set enable_Adjacency_OSPFv3 0
+            set enable_Adjacency_OLSRv2 0
+            set adjacency_config(proto) "OLSR_proto"
+	} elseif { $command == "menu5" && $enable_Adjacency_OLSRv2 } {
+            set enable_Adjacency_OSPFv2 0
+            set enable_Adjacency_OSPFv3 0
+            set enable_Adjacency_OLSR 0
+            set adjacency_config(proto) "OLSRv2_proto"
+        } 
     }
 
     # Initialize
     if { $enable_Adjacency && $command != "stop" } {
 	array unset adjacency_cache *
 	foreach node $node_list { ;# save router-id node pairs for later lookup
-	    if { [nodeType $node] != "router" } { continue }
-	    if {[lsearch [getNodeServices $node true] "zebra"] < 0} {
+            if { [nodeType $node] != "router" } { continue }
+	    if {[lsearch [getNodeServices $node true] "zebra"] < 0 &&
+                [lsearch [getNodeServices $node true] "OLSR"] < 0 && 
+                [lsearch [getNodeServices $node true] "OLSRv2"] < 0} {
 		continue
 	    }
 	    set rtrid [get_router_id $node]
@@ -1973,8 +2017,10 @@ proc widget_adjacency_init {command} {
     # De-initialize
     if { !$enable_Adjacency || $command == "stop" } {
 	set enable_Adjacency 0
-	set enable_Adjacency_v2 0
-	set enable_Adjacency_v3 0
+	set enable_Adjacency_OSPFv2 0
+	set enable_Adjacency_OSPFv3 0
+	set enable_Adjacency_OLSR 0
+	set enable_Adjacency_OLSRv2 0
 	$c delete -withtags "adjline"
 	after 200 { .c delete -withtags "adjline" }
     }
@@ -1985,25 +2031,56 @@ proc widget_adjacency_init {command} {
 #
 proc widget_adjacency_periodic { now } {
     global node_list adjacency_config adjacency_cache adjacency_lock
-    global enable_Adjacency curcanvas
+    global enable_Adjacency curcanvas vtysh
     set changed 0
 
     set proto $adjacency_config(proto)
+    if { $proto == "OLSR_proto" } {
+        foreach node $node_list {
+	    if { [nodeType $node] != "router" } { continue }
+            if { [getNodeCanvas $node] != $curcanvas } { continue }
+            if {[lsearch [getNodeServices $node true] "OLSR"] < 0} {
+	        continue
+	    }
+            if { $adjacency_lock == $node } { continue }
+            # when using daemon, send Execute Message and draw line using
+            # widget_adjacency_callback after the response has been received
+            set sock [lindex [getEmulPlugin $node] 2]
+            set exec_num [newExecCallbackRequest adjacency]
+            set cmd "nrlConsole.py ${node}_olsr n"
+            sendExecMessage $sock $node $cmd $exec_num 0x30
+        }
+    } elseif { $proto == "OLSRv2_proto" } {
+        foreach node $node_list {
+	    if { [nodeType $node] != "router" } { continue }
+            if { [getNodeCanvas $node] != $curcanvas } { continue }
+            if {[lsearch [getNodeServices $node true] "OLSRv2"] < 0} {
+	        continue
+	    }
+            if { $adjacency_lock == $node } { continue }
+            # when using daemon, send Execute Message and draw line using
+            # widget_adjacency_callback after the response has been received
+            set sock [lindex [getEmulPlugin $node] 2]
+            set exec_num [newExecCallbackRequest adjacency]
+            set cmd "nrlConsole.py ${node}_olsrv2 n"
+            sendExecMessage $sock $node $cmd $exec_num 0x30
+        }
+    } else {
+        foreach node $node_list {
+	    if { [nodeType $node] != "router" } { continue }
+            if { [getNodeCanvas $node] != $curcanvas } { continue }
+            if {[lsearch [getNodeServices $node true] "zebra"] < 0} {
+	        continue
+	    }
 
-    foreach node $node_list {
-	if { [nodeType $node] != "router" } { continue }
-	if { [getNodeCanvas $node] != $curcanvas } { continue }
-	if {[lsearch [getNodeServices $node true] "zebra"] < 0} {
-	    continue
-	}
-
-	if { $adjacency_lock == $node } { continue }
-	# when using daemon, send Execute Message and draw line using
-	# widget_adjacency_callback after the response has been received
-	set sock [lindex [getEmulPlugin $node] 2]
-	set exec_num [newExecCallbackRequest adjacency]
-	set cmd "vtysh -c 'show $proto neighbor'"
-	sendExecMessage $sock $node $cmd $exec_num 0x30
+            if { $adjacency_lock == $node } { continue }
+            # when using daemon, send Execute Message and draw line using
+            # widget_adjacency_callback after the response has been received
+            set sock [lindex [getEmulPlugin $node] 2]
+            set exec_num [newExecCallbackRequest adjacency]
+            set cmd "$vtysh -c 'show $proto neighbor'"
+            sendExecMessage $sock $node $cmd $exec_num 0x30
+        }
     }
 }
 
@@ -2014,14 +2091,16 @@ proc exec_adjacency_callback { node execnum cmd result status } {
     global g_api_exec_num
     set changed 0
     set c .c
-
+    
     set proto $adjacency_config(proto)
     array set colors $adjacency_config(colors)
     if { $adjacency_config(offset) } { set o 5 } else { set o 0 }
 
     $c addtag adjdelete withtag "adjline && $node" ;# flag del all adjlines 
     set adjs [getadj_from_neighbors $result $proto]
+     
     foreach adj $adjs {
+            
 	    set peer [lindex $adj 0]
 	    set line [$c find withtag "adjline && $node && $peer"]
 
@@ -2048,7 +2127,7 @@ proc exec_adjacency_callback { node execnum cmd result status } {
 		set a [$c create line $x $y [expr {$o + $x + (($x2 - $x)/2) }] \
 				     [expr {$o + $y + (($y2 - $y)/2) }] \
 			 -fill $color -width 3 \
-			 -tags "adjline $node $peer"]
+			 -tags "adjline $node $peer peer_$node2"]
 		$c lower $a "node && $node"
 	    } else {; 			# update existing half line
 		$c itemconfigure $line -fill $color ;# update the color
@@ -2063,23 +2142,54 @@ proc exec_adjacency_callback { node execnum cmd result status } {
 proc exec_adjacencyrouterid_callback { node execnum cmd result status } {
     global adjacency_cache
 
-    # match both OSPFv2 and OSPFv3 responses
-    set rid [regexp -inline {Router[- ]ID[:]? [0-9]+\.[0-9]+\.[0-9]+\.[0-9]+} \
-    		$result]
-    if {$rid != ""} {
-	set rtrid [eval lindex $rid end]
-	array set adjacency_cache [list $rtrid $node]
+    #check if olsr or olsrv2 are running
+    if {[lsearch [getNodeServices $node true] "OLSR"] != -1 ||
+        [lsearch [getNodeServices $node true] "OLSRv2"] != -1 } {
+        set lines [split $result "\n"]
+        set rtrid [lindex $lines 1]
+        array set adjacency_cache [list $rtrid $node]
+    } else {
+        # match both OSPFv2 and OSPFv3 responses
+        set rid [regexp -inline {Router[- ]ID[:]? [0-9]+\.[0-9]+\.[0-9]+\.[0-9]+} \
+    			$result]
+        if {$rid != ""} {
+		set rtrid [eval lindex $rid end]
+		array set adjacency_cache [list $rtrid $node]
+        }
     }
 }
 
 # helper to convert neighbor list into adjacencies list
 proc getadj_from_neighbors { raw_input proto } {
     set ret { }
+#OLSR conversions
+    if { $proto == "OLSRv2_proto" || $proto == "OLSR_proto"} {
+        foreach line [split $raw_input "\n"] {
+            if { $line == "neighbors" } { continue }
+            if { $line == "end-neighbors" } { continue }
+            if { $line == "" } { continue }
+            set fields [split $line " "]
+            lassign $fields rtrid state metric mpr
+            # convert to OSPFv3 to simplify coloring
+            switch -exact -- "$state" {
+            "SYM"     { set state "Full" }
+            "ASYM"    { set state "Init" }
+            "LOST"    { set state "Down" }
+            "MPR"     { set state "Full" }
+            "PENDING" { set state "Init" }
+            "INVALID" { set state "Down" } 
+            }
+            lappend ret [list $rtrid $state]
+        }
+        return $ret
+    }
+#OSPF converstion
+
 #Neighbor ID     Pri    DeadTime  State/IfState         Duration I/F[State]
 #10.0.0.2          1    00:00:06   Init/PointToPoint    00:00:00 eth0[PointToP
 #10.0.0.2          1    00:00:06 Twoway/PointToPoint    00:00:00 eth0[PointToP
 #10.0.0.2          1    00:00:06   Full/PointToPoint    00:00:38 eth0[PointToP
-#10.0.7.2          1 Full/Backup       37.240s 10.0.0.2        eth0:10.0.0.1
+#10.0.7.2          1 Full/Backup       37.240s 10.0.0.2        eth0:10.0.0.1    
     foreach line [split $raw_input "\n"] {
 	set rtrid [string trim [string range $line 0 14]]
 	if { $rtrid == "Neighbor ID" } { continue }
@@ -2127,18 +2237,13 @@ proc widget_adjacency_move { c node done } {
 			     [expr {$o + $y + (($y2 - $y)/2) }]
 	$c lower $line "node && $n1"
 	# move any half line coming from peer to this moved node
-	foreach peerline [$c find withtag "adjline && $n2"] {
-	    set peer2 [lindex [$c gettags $peerline] 2]
-	    if { ![info exists adjacency_cache($peer2)] } { continue }
-	    if { $adjacency_cache($peer2) == $node } {
-		$c coords $peerline $x2 $y2 \
-			[expr {$o + $x2 + (($x - $x2)/2) }] \
-			[expr {$o + $y2 + (($y - $y2)/2) }]
-		$c lower $peerline "node && $n2"
-		break
-	    }
-
-	} ;# end foreach peerline
+	foreach peerline [$c find withtag "adjline && $n2 && peer_$n1"] {
+	    set peer2 [lindex [$c gettags $peerline] 1]
+	    $c coords $peerline $x2 $y2 \
+		[expr {$o + $x2 + (($x - $x2)/2) }] \
+		[expr {$o + $y2 + (($y - $y2)/2) }]
+	    $c lower $peerline "node && $n2"
+        }
     } ;# end foreach line
 
     if { $done } { set adjacency_lock 0 }
@@ -2153,12 +2258,31 @@ proc widget_adjacency_init_submenu { m } {
     $m add cascade -label "Adjacency" -menu $m.adj
     set w "Adjacency"
 
-    foreach v [list 2 3] {
-	global enable_${w}_v${v}
-	set enable_${w}_v${v} 0
-	$m.adj add checkbutton -label "OSPFv$v" -variable enable_${w}_v${v} \
-		-command "[lindex $widgets($w) 1] menu$v"
-    }
+#    foreach v [list 2 3] {
+#	global enable_${w}_v${v}
+#	set enable_${w}_v${v} 0
+#	$m.adj add checkbutton -label "OSPFv$v" -variable enable_${w}_v${v} \
+#		-command "[lindex $widgets($w) 1] menu$v"
+#    }
+    global enable_Adjacency_OSPFv2
+    set enable_Adjacency_OSPFv2 0
+    $m.adj add checkbutton -label "OSPFv2" -variable enable_Adjacency_OSPFv2 \
+        -command "[lindex $widgets(Adjacency) 1] menu2"
+    
+    global enable_Adjacency_OSPFv3
+    set enable_Adjacency_OSPFv3 0
+    $m.adj add checkbutton -label "OSPFv3" -variable enable_Adjacency_OSPFv3 \
+        -command "[lindex $widgets(Adjacency) 1] menu3"
+    
+    global enable_Adjacency_OLSR
+    set enable_Adjacency_OLSR 0
+    $m.adj add checkbutton -label "OLSR" -variable enable_Adjacency_OLSR \
+        -command "[lindex $widgets(Adjacency) 1] menu4"
+    
+    global enable_Adjacency_OLSRv2
+    set enable_Adjacency_OLSRv2 0
+    $m.adj add checkbutton -label "OLSRv2" -variable enable_Adjacency_OLSRv2 \
+        -command "[lindex $widgets(Adjacency) 1] menu5"
 }
 
 # load the widgets.conf file when this file is loaded
